/**
 * MEDOR: Middleware Enabling Distributed Object Requests
 *
 * Copyright (C) 2001-2003 France Telecom R&D
 * Contact: alexandre.lefebvre@rd.francetelecom.com
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Initial developers: M. Alia, A. Lefebvre, S. Chassande-Barrioz
 */

package org.objectweb.medor.query.rdb.lib;

import org.objectweb.medor.api.MedorException;
import org.objectweb.medor.api.QueryLeafException;
import org.objectweb.medor.api.Field;
import org.objectweb.medor.datasource.api.DataStore;
import org.objectweb.medor.expression.api.ParameterOperand;
import org.objectweb.medor.lib.Log;
import org.objectweb.medor.query.api.NestQueryNode;
import org.objectweb.medor.query.api.NestedField;
import org.objectweb.medor.query.api.OptimizationMetaData;
import org.objectweb.medor.query.api.QueryTreeField;
import org.objectweb.medor.query.lib.Nest;
import org.objectweb.medor.query.rdb.api.RdbExpField;
import org.objectweb.medor.query.rdb.api.RdbQueryLeaf;
import org.objectweb.medor.query.rdb.api.RdbExpQueryLeaf;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

import java.util.ArrayList;

/**
 * This class represents a QueryLeaf which contains aggregate functions.
 * <p> It is built on top of a RbdQueryLeaf.
 */
public class AggregateRdbQueryNode extends Nest
		implements RdbQueryLeaf, NestQueryNode {

	DataStore ds;
	String nodeName;
	RdbExpField groupByField;
	RdbExpQueryLeaf subRdbQL;
	String query = null;

	protected Logger log = null;
	protected boolean debug = false;
	private boolean isSubquery = false;

	/**
	 * Constructs a RDB QueryNode for an aggregation.
	 * <p>The SQL request (string) will be calculated.
	 * @param ds is the DataStore associated to the QueryLeaf to be created
	 * @throws QueryLeafException if the QualifiedTables have all the same
	 * names, and in this case also the same aliases.
	 */
	public AggregateRdbQueryNode(QueryTreeField[] groupedFields,
                String groupedFieldName,
                QueryTreeField[] groupByFields,
				BasicRdbExpQueryLeaf subQL,
                OptimizationMetaData optimizationMD, DataStore ds,
								String nodeName)
			throws MedorException {
		super(groupedFields, groupedFieldName, groupByFields, nodeName,
				optimizationMD);
		log = Log.loggerFactory.getLogger(
				Log.MEDOR_PREFIX + ".query.rdb");
		debug = log != null && log.isLoggable(BasicLevel.DEBUG);
		this.nodeName = nodeName;
		this.ds = ds;
		this.subRdbQL = subQL;
		//removing the nested field
		removeField(groupedFieldName);
	}

	public String getName() {
		return nodeName;
	}

	public String getSqlRequest(ParameterOperand[] pos, ArrayList al)
			throws MedorException {
		if (al == null) {
			return getSqlRequest(pos);
		} else {
			throw new MedorException("Request with selectList not supported");
		}
	}

	/**
	 * Builds and returns the SQL query as a String corresponding to the
	 * current RdbExpQueryLeaf.
	 * <p>
	 * The SQL query is computed using the aggregate fields and relies on
	 * the child RdbQueryLeaf.
	 * @return the SQL query as a String.
	 */
	public String getSqlRequest(ParameterOperand[] pos) throws MedorException {
		ArrayList selectFields = new ArrayList();
		for (int i = 0; i < fields.size(); i++) {
			if (!(fields.get(i) instanceof NestedField)) {
				selectFields.add(fields.get(i));
			}
		}
		if (query != null) {
			return query;
		}
		if (subRdbQL.isSubquery()) {
			//Constructs a subquery only if there is a count on multiple fields.
			//Note: in this case, no other aggregates are supported.
			boolean subRdbQLDistinct = subRdbQL.getDistinct();
			subRdbQL.setDistinct(this.getDistinct());
			query = subRdbQL.getSelectList("SELECT ", selectFields, false);
			subRdbQL.setDistinct(subRdbQLDistinct);

			query += " FROM (";
			query += subRdbQL.getSqlRequest(pos, null);
			query += ")";
			if (subRdbQL.getRdbAdapter() != null) {
				query += subRdbQL.getRdbAdapter().getSubqueryAlias();
			}
		} else {
			query = subRdbQL.getSqlRequest(pos, selectFields);
		}
		return query;
	}

	public String getSelectList(String selectList, ArrayList selectFields,
								boolean qualified)
		throws MedorException {
		throw new QueryLeafException("Unsupported method getSelectList for AggregateRdbQueryNode");
	}

	public DataStore getDataStore() {
		return ds;
	}

	public void setIsSubquery(boolean subquery) {
		isSubquery = subquery;
	}

	public boolean isSubquery() {
		return isSubquery;
	}


}